import '../cache.dart';
import 'base_store.dart';

/// A [CacheStore] that keeps all its values in memory.
class MemoryCacheStore<T> extends BaseCacheStore<T> {
  final List<Cache<T>> _items = [];

  @override
  Future<void> clean(CachePriority priority) {
    _items.removeWhere((x) => x.priority.index < priority.index);
    return Future.value();
  }

  @override
  Future<Cache<T>> read(String key) {
    return Future.value(_items.firstWhere((x) => x.key == key, orElse:() => null));
  }

  @override
  Future<List<Cache<T>>> readAll(List<String> keys) {
    return Future.value(_items.where((x) => keys.contains(x.key)).toList());
  }

  @override
  Future<void> set(Cache<T> value) {
    _items.removeWhere((x) => x.key == value.key);
    _items.insert(0,value);
    return Future.value();
  }

  @override
  Future<void> setAll(List<Cache<T>> values) {
    _items.removeWhere((x) => values.any((v) => x.key == v.key));
    _items.insertAll(0,values);
    return Future.value();
  }

  @override
  Future<void> delete(String key) {
    _items.removeWhere((x) => x.key == key);
    return Future.value();
  }
}
